==> simplerllm-0.3.0.1.tar.gz: simplerllm-0.3.0.1/tests/test_chat_youtube.py <==
import asyncio
from typing import List, Dict, Any
import numpy as np
import nltk
from nltk.tokenize import sent_tokenize
from SimplerLLM.vectors.vector_storage import VectorDatabase, SerializationFormat
from SimplerLLM.language.embeddings import EmbeddingsLLM as EmbeddingLLM, EmbeddingsProvider
from SimplerLLM.language.llm import LLM as GenerationLLM, LLMProvider

# Download necessary NLTK data
nltk.download('punkt')

# Initialize the embedding and generation models
embedding_llm = EmbeddingLLM.create(provider=EmbeddingsProvider.OPENAI, model_name="text-embedding-3-small")
generation_llm = GenerationLLM.create(provider=LLMProvider.OPENAI, model_name="gpt-3.5-turbo")



def split_text_into_chunks(text: str, chunk_size: int = 3) -> List[str]:
    sentences = sent_tokenize(text)
    chunks = []
    current_chunk = []
    current_length = 0
    
    for sentence in sentences:
        if current_length + len(sentence.split()) > chunk_size:
            chunks.append(" ".join(current_chunk))
            current_chunk = [sentence]
            current_length = len(sentence.split())
        else:
            current_chunk.append(sentence)
            current_length += len(sentence.split())
    
    if current_chunk:
        chunks.append(" ".join(current_chunk))
    
    return chunks

async def initialize_database(db_folder: str) -> VectorDatabase:
    db = VectorDatabase(db_folder, use_semantic_connections=True)
    await db.load_from_disk("sample_text", SerializationFormat.JSON)
    return db

def create_embedding(text: str) -> np.ndarray:
    response = embedding_llm.generate_embeddings(text)
    # The OpenAI API returns a list of embedding objects
    # We assume there's only one embedding for the input text
    embedding = response[0].embedding
    return np.array(embedding)


async def add_text_to_database(db: VectorDatabase, text: str) -> None:
    chunks = split_text_into_chunks(text)
    for i, chunk in enumerate(chunks):
        if chunk.strip():  # This checks if the chunk is not empty or just whitespace
            embedding = create_embedding(chunk)
            db.add_vector(chunk, embedding, {"index": i})
    await db.save_to_disk("sample_text", SerializationFormat.JSON)

def generate_response(query: str, context: List[str]) -> str:
    prompt = f"""
    Given the following context and query, provide a concise and relevant answer. If the context doesn't contain enough information to answer the query, say so.

    Context:
    {' '.join(context)}

    Query: {query}

    Answer:
    """
    return generation_llm.generate_response(prompt=prompt)

async def chat_with_database(db: VectorDatabase) -> None:
    print("Chatbot: Hello! I'm ready to answer questions about the sample text. Type 'exit' to quit.")
    while True:
        user_input = input("You: ")
        if user_input.lower() == 'exit':
            print("Chatbot: Goodbye!")
            break
        
        query_embedding = create_embedding(user_input)

        results = db.semantic_search(query_embedding, top_k=3, depth=1)
        
        if results:
            context = [result['chunk_text'] for result in results]
            response = generate_response(user_input, context)
            print(f"Chatbot: {response}")
        else:
            print("Chatbot: I'm sorry, I couldn't find any relevant information to answer your question.")

async def main():
    db_folder = "vector_db"
    sample_text = """
    The Python programming language, created by Guido van Rossum in the late 1980s, has become one of the most popular languages in the world. It is known for its simplicity and readability, making it an excellent choice for beginners and experts alike. Python is versatile and can be used for web development, data analysis, artificial intelligence, and more.

    One of Python's key features is its extensive standard library, which provides a wide range of modules and functions for various tasks. This "batteries included" philosophy means that developers can often find the tools they need without having to rely on external libraries.

    Python's popularity in data science and machine learning has grown significantly in recent years. Libraries such as NumPy, Pandas, and Scikit-learn have made it easier for researchers and data scientists to work with large datasets and implement complex algorithms.

    The language's support for object-oriented, functional, and procedural programming paradigms makes it flexible and suitable for a variety of programming styles. This flexibility, combined with its clean syntax, has contributed to Python's widespread adoption in both industry and academia.
    """
    
    db = await initialize_database(db_folder)
    await add_text_to_database(db, sample_text)
    await chat_with_database(db)

if __name__ == "__main__":
    asyncio.run(main())

==> simplerllm-0.3.0.4.tar.gz: simplerllm-0.3.0.4/tests/test_chat_youtube.py <==
import asyncio
from typing import List, Dict, Any
import numpy as np
import nltk
from nltk.tokenize import sent_tokenize
from SimplerLLM.vectors.vector_storage import VectorDatabase, SerializationFormat
from SimplerLLM.language.embeddings import EmbeddingsLLM as EmbeddingLLM, EmbeddingsProvider
from SimplerLLM.language.llm import LLM as GenerationLLM, LLMProvider

# Download necessary NLTK data
nltk.download('punkt')

# Initialize the embedding and generation models
embedding_llm = EmbeddingLLM.create(provider=EmbeddingsProvider.OPENAI, model_name="text-embedding-3-small")
generation_llm = GenerationLLM.create(provider=LLMProvider.OPENAI, model_name="gpt-3.5-turbo")


def split_text_into_chunks(text: str, chunk_size: int = 3) -> List[str]:
    sentences = sent_tokenize(text)
    chunks = []
    current_chunk = []
    current_length = 0
    
    for sentence in sentences:
        if current_length + len(sentence.split()) > chunk_size:
            chunks.append(" ".join(current_chunk))
            current_chunk = [sentence]
            current_length = len(sentence.split())
        else:
            current_chunk.append(sentence)
            current_length += len(sentence.split())
    
    if current_chunk:
        chunks.append(" ".join(current_chunk))
    
    return chunks

def create_embedding(text: str) -> np.ndarray:
    response = embedding_llm.generate_embeddings(text)
    # The OpenAI API returns a list of embedding objects
    # We assume there's only one embedding for the input text
    embedding = response[0].embedding
    return np.array(embedding)

async def add_text_to_database(db: VectorDatabase, text: str) -> None:
    chunks = split_text_into_chunks(text)
    for i, chunk in enumerate(chunks):
        if chunk.strip():  # This checks if the chunk is not empty or just whitespace
            embedding = create_embedding(chunk)
            db.add_vector(chunk, embedding, {"index": i})
    await db.save_to_disk(SerializationFormat.BINARY)

def generate_response(query: str, context: List[str]) -> str:
    prompt = f"""
    Given the following context and query, provide a concise and relevant answer. If the context doesn't contain enough information to answer the query, say so.

    Context:
    {' '.join(context)}

    Query: {query}

    Answer:
    """
    return generation_llm.generate_response(prompt=prompt)

async def chat_with_database(db: VectorDatabase) -> None:
    print("Chatbot: Hello! I'm ready to answer questions about the sample text. Type 'exit' to quit.")
    while True:
        user_input = input("You: ")
        if user_input.lower() == 'exit':
            print("Chatbot: Goodbye!")
            break
        
        query_embedding = create_embedding(user_input)

        results = db.semantic_search(query_embedding, top_k=3, depth=1)
        
        if results:
            context = [result['text'] for result in results]
            response = generate_response(user_input, context)
            print(f"Chatbot: {response}")
        else:
            print("Chatbot: I'm sorry, I couldn't find any relevant information to answer your question.")

async def main():
    
    sample_text = """
    The Python programming language, created by Guido van Rossum in the late 1980s, has become one of the most popular languages in the world. It is known for its simplicity and readability, making it an excellent choice for beginners and experts alike. Python is versatile and can be used for web development, data analysis, artificial intelligence, and more.

    One of Python's key features is its extensive standard library, which provides a wide range of modules and functions for various tasks. This "batteries included" philosophy means that developers can often find the tools they need without having to rely on external libraries.

    Python's popularity in data science and machine learning has grown significantly in recent years. Libraries such as NumPy, Pandas, and Scikit-learn have made it easier for researchers and data scientists to work with large datasets and implement complex algorithms.

    The language's support for object-oriented, functional, and procedural programming paradigms makes it flexible and suitable for a variety of programming styles. This flexibility, combined with its clean syntax, has contributed to Python's widespread adoption in both industry and academia.
    """
    
    db = VectorDatabase("test_db", use_semantic_connections=True)
    await add_text_to_database(db, sample_text)
    await chat_with_database(db)

if __name__ == "__main__":
    asyncio.run(main())
